@@ -9,7 +9,6 @@ from asyncio import run as aiorun
 from datafiles import datafile, formats
 from datafiles import Missing
 from pydoc import locate
-import questionary
 import random
 import json
 import os
@@ -31,7 +30,6 @@ else:
 
 
 
-
 # constants for no (0.0), low (0.4), normal (0.7), high (0.9), and max (1.0) temperature:
 @dataclass
 class TEMPERATURE:
@@ -223,8 +221,8 @@ class Completion:
 
     # post init to initialize the prompt and params
     def load_all(self):
-       self.petition = Petition.objects.get(self.petition_name) if self.petition_name is not Missing else None
-       self.parent = Completion.objects.get(self.parent_name) if self.parent_name is not Missing else None
+       self.petition = Petition.objects.get(self.petition_name) if self.petition_name is not Missing and self.petition_name is not None else None
+       self.parent = Completion.objects.get(self.parent_name) if self.parent_name is not Missing and self.parent_name is not None else None
 
 
 @datafile(PLUNKYLIB_BASE_DIR + "/vectorsearchparams/{self.name}.yml", defaults=True)
@@ -393,46 +391,6 @@ class RandomListGetter:
         return random.choice(NamedList(k, Missing).items)
     __getattr__ = __getitem__
 
-class QuestionGetter:
-    """Used for interactive mode to prompt the user for values when completing a prompt/petition"""
-    def __init__(self, src, func):
-        self.src = src
-        self.func = func
-
-    def __getitem__(self, k):
-        tosearch = self.src
-        context = ''
-        eol = False
-        # ask them if they want to complete the petition or type it themselves
-        # using questionary.select:
-        choice = questionary.select(f"{k}: Do you want to complete the petition {k} with GPT-3?",
-            choices=["Yes, autogenerate", "No, I'll type it myself"]).ask()
-        if choice == "Yes, autogenerate":
-            async def completer_coro():
-                return await self.func(k)
-            x = aiorun(completer_coro())
-            return x
-        else:
-            # loop through every line looking for k inside of it:
-            for line in tosearch.split("\n"):
-                if "." + k in line:
-                    context = line
-                    if line.endswith(k+"}") and not tosearch.endswith(k+"}"):
-                        eol = True
-                    break
-
-            # ask them if what they want to use to replace "k"?
-            x = questionary.text(f"\nFor this line:\n   {context}\nWhat do you want to use to replace 'question.{k}'?").ask()
-            
-            if eol and len(x) > 1:
-                x = x + "\n"
-            # print x and tell them what we're doing
-            print(f"{k} produced:\n{x}---")
-            return x
-    
-    __getattr__ = __getitem__
-
-
 
 def _trimmed_fetch_response(resp, n):
     # resp.json()['completions'][0]['data']['text']
